////////////////////////////////////////////////////////////////////////////////
//
// Fracter - A simple Mandelbrot Set viewer.
//
// Copyright (C) 2010 Sarah Eris Horsley Caffee
//
// This file is part of Fracter.
//
// Fracter is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
//
// Fractal.h
// Fractal calculator virtual class definition.  Actual fractals should 
// implement this.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef FRACTAL_H
#define FRACTAL_H

#include <map>
#include <string>
#include <typeinfo>

#include "SDL.h"

////////////////////////////////////////////////////////////////////////////////
class View
   {
   public:
   View(std::string t, 
	double re, double im, double s, 
	unsigned int max);
   virtual ~View();

   std::string const & get_type() const;


   private:
   friend class Fractal;

   View();
   View & operator=(const View &);

   std::string type;
   double re_min, im_max, size;
   unsigned int max_iter;
   };


////////////////////////////////////////////////////////////////////////////////
class Fractal
   {
   public:
   Fractal();
   virtual ~Fractal();

   virtual bool calc_set() = 0;
   virtual bool init(SDL_Surface * surf);

   double get_re_min() const;
   double set_re_min(const double r_min);

   double get_im_max() const;
   double set_im_max(const double i_max);

   double get_size() const;
   double set_size(const double s);

   unsigned int get_max_iter() const;
   unsigned int set_max_iter(const unsigned int iter);
   unsigned int inc_max_iter();
   unsigned int dec_max_iter();

   void set_calc_needed();

   virtual std::string const & get_display_name() const;
   virtual std::string const & get_fractal_name() const;


   virtual bool get_option(std::string option, void * value) const;
   virtual bool set_option(std::string option, void * value);

   virtual View * save_view();
   virtual void restore_view(View * v);

   protected:

   virtual void draw_pixel(int x, int y, Uint32 * color);
   
   // fractal_name is the basic name of ths fractal, such as "Mandelbrot set"
   //   or "Julia set"
   // display_name is probably the same, but might have a bit of extra info,
   //   such as "Julia set K=(1,2)
   std::string display_name;
   std::string fractal_name;
   

   // The dimensions in the complex plane of the calculated image.  
   // Square image only with die of length "size".
   // Upper left corner of the region is (re_min, im_max)
   double re_min;
   double im_max;
   double size;

   // Number of iterations to perform when calculating.
   unsigned int max_iter;

   // Set to true if the image needs to be recalculated.
   bool calc_needed;
   
   // Our drawing surface.
   SDL_Surface * surface;

   private:
   // Disallow copy constructor and operator=
   Fractal(const Fractal &);
   Fractal & operator=(const Fractal &);
   };



////////////////////////////////////////////////////////////////////////////////
// Abstract Factory class.
// A singleton, but it's not thread safe.
class FractalFactory
   {
   public:
   virtual ~FractalFactory();
   virtual Fractal * create() const = 0;
   };

std::map <std::string, FractalFactory *> & get_Fractal_map();

#endif
